@@ -30,7 +30,7 @@
 
 import socket
 from abc import ABC
-from typing import Optional, Set
+from typing import Optional, Iterable
 
 from startrek.types import Address, AddressPairMap
 from startrek import Channel
@@ -79,16 +79,17 @@ class PackageHub(BaseHub, ABC):
             channel = self._create_channel(remote=None, local=address, sock=sock)
             self._set_channel(channel=channel)
 
+    #
+    #   Channel
+    #
+
     # noinspection PyMethodMayBeStatic
     def _create_channel(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket) -> Channel:
         # override for user-customized channel
         return PackageChannel(remote=remote, local=local, sock=sock)
 
-    def put_channel(self, channel: Channel):
-        self._set_channel(channel=channel)
-
     # Override
-    def _all_channels(self) -> Set[Channel]:
+    def _all_channels(self) -> Iterable[Channel]:
         """ get a copy of all channels """
         return self.__channel_pool.items
 
@@ -111,7 +112,8 @@ class PackageHub(BaseHub, ABC):
             # get any channel
             channels = self._all_channels()
             for sock in channels:
-                if sock is not None:
+                address = sock.remote_address
+                if address is None or address == remote:
                     return sock
             # channel not found
         else:
